import cv2
import mediapipe as mp
import numpy as np
import os

class SelfieSegmentation():
    def __init__(self, BG_COLOR=(255, 255, 255), MASK_COLOR=(0, 0, 0)):
        self.mp_drawing = mp.solutions.drawing_utils
        self.mp_selfie_segmentation = mp.solutions.selfie_segmentation
        self.BG_COLOR = BG_COLOR
        self.MASK_COLOR = MASK_COLOR

    def staticSegmentation(self, path, saveto, saveMask=0, backgroundimage=0):
        with self.mp_selfie_segmentation.SelfieSegmentation(
                model_selection=0) as selfie_segmentation:
            listImg = os.listdir(path)
            i = 10
            for imgPath in listImg:
                image = cv2.imread(f'{path}/{imgPath}')
                image_height, image_width, _ = image.shape
                results = selfie_segmentation.process(cv2.cvtColor(image, cv2.COLOR_BGR2RGB))
                condition = np.stack((results.segmentation_mask,) * 3, axis=-1) > 0.1
                fg_image = np.zeros(image.shape, dtype=np.uint8)
                fg_image[:] = self.MASK_COLOR
                if backgroundimage == 0:
                    bg_image = np.zeros(image.shape, dtype=np.uint8)
                    bg_image[:] = self.BG_COLOR
                else:
                    img = cv2.imread(backgroundimage)
                    bg_image = img
                output_image = np.where(condition, fg_image, bg_image)
                abspath = saveto + str(i) + '.png'
                cv2.imwrite(abspath, output_image)
                if saveMask:
                    abspath = saveto + 'mask' + str(i) + '.png'
                    cv2.imwrite(abspath, output_image)
                i += 1

    def videoSegmentation(self, cam=0, threshold=0.5, backgroundimage=0):
        cap = cv2.VideoCapture(cam)
        with self.mp_selfie_segmentation.SelfieSegmentation(
                model_selection=1) as selfie_segmentation:
            bg_image = None
            while cap.isOpened():
                success, image = cap.read()
                if not success:
                    print("Ignoring empty camera frame.")
                    continue
                image = cv2.cvtColor(cv2.flip(image, 1), cv2.COLOR_BGR2RGB)
                image.flags.writeable = False
                results = selfie_segmentation.process(image)
                image.flags.writeable = True
                image = cv2.cvtColor(image, cv2.COLOR_RGB2BGR)
                condition = np.stack(
                    (results.segmentation_mask,) * 3, axis=-1) > threshold
                if backgroundimage != 0:
                    img = cv2.imread(backgroundimage)
                    bg_image = img
                else:
                    if bg_image is None:
                        bg_image = np.zeros(image.shape, dtype=np.uint8)
                        bg_image[:] = self.BG_COLOR
                output_image = np.where(condition, image, bg_image)

                cv2.imshow('MediaPipe Selfie Segmentation', output_image)
                if cv2.waitKey(5) == ord('q'):
                    break
        cap.release()



from setuptools import setup, find_packages
import codecs
import os

VERSION = '0.0.1'
DESCRIPTION = 'This package is for Selfie segmentation of image dataset'
LONG_DESCRIPTION = 'This library is used for selfie segmentation of static and live webcam feed. You ca add images or ' \
                   'a constant backgroundd colour over your image/live feed. It is usefull for projects like: ' \
                   'Face recognition, Facial expression recognition and others, where an accurate segmentation will ' \
                   'enhance the feed to the neural network therefore increasing the accuracy of the model'

# Setting up
setup(
    name="selfiesegmentation",
    version=VERSION,
    author="Sumit Singh",
    author_email="<iamsumit1904@gmail.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=['opencv-python', 'numpy', 'mediapipe'],
    keywords=['python', 'augmentation', 'image', 'data augmentation', 'image dataset'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)
